// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Stage.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ACTIVATEAPP()
	ON_WM_PALETTECHANGED()
	ON_WM_MOVE()
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_EDIT_INSERTSHAPE, OnEditInsertShape)
	ON_COMMAND(ID_EDIT_TRANSFORM, OnEditTransform)
	ON_UPDATE_COMMAND_UI(ID_EDIT_TRANSFORM, OnUpdateEditTransform)
	ON_COMMAND(ID_EDIT_PLANE, OnEditPlane)
	ON_COMMAND(ID_EDIT_TRANSLATEX, OnEditTranslatex)
	ON_COMMAND(ID_EDIT_TRANSLATEY, OnEditTranslatey)
	ON_COMMAND(ID_EDIT_TRANSLATEZ, OnEditTranslatez)
	ON_COMMAND(ID_EDIT_ROTATEX, OnEditRotatex)
	ON_COMMAND(ID_EDIT_ROTATEY, OnEditRotatey)
	ON_COMMAND(ID_EDIT_ROTATEZ, OnEditRotatez)
	ON_COMMAND(ID_EDITSCALEX, OnEditscalex)
	ON_COMMAND(ID_EDITSCALEY, OnEditscaley)
	ON_COMMAND(ID_EDITSCALEZ, OnEditscalez)
	ON_COMMAND(ID_EDIT_ROTTRAN, OnEditRottran)
	ON_COMMAND(ID_EDIT_TRANROT, OnEditTranrot)
	ON_COMMAND(ID_EDIT_RESET, OnEditReset)
	ON_COMMAND(ID_EDIT_ROBJX, OnEditRobjx)
	ON_COMMAND(ID_EDIT_ROBJY, OnEditRobjy)
	ON_COMMAND(ID_EDIT_ROBJZ, OnEditRobjz)
	ON_COMMAND(ID_EDIT_SHEAR, OnEditShear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pScene = NULL;
	m_pCurShape = FALSE;
	m_pTransDlg = NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// Create the 3D window
	if (!m_wnd3d.Create(this, IDC_3DWND)) {
		return -1;
	}

	// TODO: Remove this if you don't want tool tips or a resizeable toolbar
	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	NewScene();
	ASSERT(m_pScene);

	// Create a shape to add
	m_pCurShape  = new C3dShape;
	m_pCurShape->Load(IDX_PLANE);
	m_pScene->AddChild(m_pCurShape);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Make the initial window a fixed size
	cs.cx = 300;
	cs.cy = 350;

	return CFrameWnd::PreCreateWindow(cs);
}

BOOL CMainFrame::NewScene()
{
	// Delete any scene we might have
	if (m_pScene) {
		m_wnd3d.SetScene(NULL);
		delete m_pScene;
		m_pScene = NULL;
	}
	if (m_pCurShape) {
		delete m_pCurShape;
		m_pCurShape = NULL;
	}

	// Create an initial scene
	m_pScene = new C3dScene;
	if (!m_pScene->Create()) return FALSE;

	// Set up the lighting
    C3dDirLight dl;
    dl.Create(0.8, 0.8, 0.8);
    m_pScene->AddChild(&dl);
    dl.SetPosition(-2, 2, -5);
    dl.SetDirection(1, -1, 1);
    m_pScene->SetAmbientLight(0.4, 0.4, 0.4);
	m_wnd3d.SetScene(m_pScene);

	// Show the axes
	double rd = 0.05;
	C3dShape rz;
	rz.CreateRod(0, 0, -8, 0, 0, 8, rd);
	m_pScene->AddChild(&rz);
	C3dShape ry;
	ry.CreateRod(0, -8, 0, 0, 6, 0, rd);
	m_pScene->AddChild(&ry);
	C3dShape rx;
	rx.CreateRod(-8, 0, 0, 8, 0, 0, rd);
	m_pScene->AddChild(&rx);

	// Move the camera so we get a better view of the axes
	m_pScene->SetCameraPosition(C3dVector(10, 10, -10));
	m_pScene->SetCameraDirection(C3dVector(-1, -1, 1));


	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::RecalcLayout(BOOL bNotify) 
{
    // Rearrange the control bars and fit the 3D window in the middle
	// Let the frame rearrange the control bars
	CFrameWnd::RecalcLayout(bNotify);

	// Find the space that's left over
	CRect rc;
	RepositionBars(0,
				   0xFFFF,
				   IDC_3DWND,
				   CWnd::reposQuery,
				   &rc);
	if (IsWindow(m_wnd3d.GetSafeHwnd())) {
		m_wnd3d.MoveWindow(&rc, FALSE);
	}
}


void CMainFrame::OnDestroy() 
{
	CFrameWnd::OnDestroy();
	
	// shut down any modeless dialogs
	if (m_pTransDlg) {
		m_pTransDlg->DestroyWindow();
	}

	// Delete any scene we might have
	m_wnd3d.SetScene(NULL);
	if (m_pScene) {
		delete m_pScene;
		m_pScene = NULL;
	}
	if (m_pCurShape) {
		delete m_pCurShape;
		m_pCurShape = NULL;
	}
}

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask) 
{
	CFrameWnd::OnActivateApp(bActive, hTask);
	
	// Tell the 3D window about the new state
	m_wnd3d.SendMessage(WM_ACTIVATEAPP,
						(WPARAM)bActive,
						(LPARAM)hTask);
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd) 
{
	// Let the 3D window know
	m_wnd3d.SendMessage(WM_PALETTECHANGED,
					   pFocusWnd ? (WPARAM)pFocusWnd->GetSafeHwnd() : 0);
}

void CMainFrame::OnMove(int x, int y) 
{
	CFrameWnd::OnMove(x, y);
	
	// Let the 3D window know the frame has moved
	m_wnd3d.SendMessage(WM_MOVE, 0, MAKELPARAM(0, 0));
}

void CMainFrame::OnFileNew() 
{
	NewScene();
}

BOOL CMainFrame::OpenFile(const char* pszPath)
{
	// Create a new scene
	NewScene();
	ASSERT(m_pScene);

	// Try for a shape file
	m_pCurShape = new C3dShape;
	const char* pszFile = m_pCurShape->Load(pszPath);
	if (!pszFile) return FALSE;

	// Attach the new shape to the scene
	m_pScene->AddChild(m_pCurShape);

	// Add the name to the recent file list
	AfxGetApp()->AddToRecentFileList(pszFile);
	Status("Opened: %s", pszFile);
	return TRUE;
}

void CMainFrame::OnFileOpen() 
{
	OpenFile(NULL);
}

void CMainFrame::OnEditInsertShape() 
{
	CShapeDlg dlg("Insert Shape", this);

	if (dlg.DoModal() != IDOK) return;

	// Add the new shape
	NewScene();
	if (dlg.m_pShape) {
		ASSERT(m_pScene);
		m_pScene->AddChild(dlg.m_pShape);

		if (m_pCurShape) delete m_pCurShape;
		m_pCurShape = dlg.m_pShape;
	}
}

void CMainFrame::Status(const char* pszFormat, ...)
{
	char buf[256];
	va_list arglist;
	va_start(arglist, pszFormat);
    vsprintf(buf, pszFormat, arglist);
	va_end(arglist);
	m_wndStatusBar.SetWindowText(buf);
	m_wndStatusBar.UpdateWindow();
}

/////////////////////////////////////////////////////////////
// Transform dialog

// callback from the dialog	when it closes
// static
void CMainFrame::OnCloseTrfmDlg(void* pArg)
{
	CMainFrame* pThis = (CMainFrame*) pArg;
	ASSERT(pThis);
	ASSERT(pThis->IsKindOf(RUNTIME_CLASS(CMainFrame)));
	pThis->m_pTransDlg = NULL;
}

// callback from the dialog to get the frame
// static
C3dFrame* CMainFrame::OnGetFrame(void* pArg)
{
	CMainFrame* pThis = (CMainFrame*) pArg;
	ASSERT(pThis);
	ASSERT(pThis->IsKindOf(RUNTIME_CLASS(CMainFrame)));
	return pThis->m_pCurShape;
}

void CMainFrame::ShowTrfmDlg()
{
	ASSERT(m_pCurShape);
	if (m_pTransDlg) {
		m_pTransDlg->BringWindowToTop();
	} else {
		m_pTransDlg = new CTransformDlg("Transforms",
										OnCloseTrfmDlg,
										OnGetFrame,
										this);
	}
}

void CMainFrame::OnEditTransform() 
{
	ShowTrfmDlg();
}

void CMainFrame::OnUpdateEditTransform(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pCurShape != NULL);
}

// Insert an airplane
void CMainFrame::OnEditPlane() 
{
	NewScene();
	m_pCurShape = new C3dShape;
	m_pCurShape->Load(IDX_PLANE);
	m_pScene->AddChild(m_pCurShape);
}

void CMainFrame::OnEditTranslatex() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Translate(2, 0, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditTranslatey() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Translate(0, 2, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditTranslatez() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Translate(0, 0, 2);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditRotatex() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(45, 0, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditRotatey() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(0, 45, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditRotatez() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(0, 45, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditscalex() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Scale(2.0, 1.0, 1.0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditscaley() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Scale(1.0, 2.0, 1.0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditscalez() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Scale(1.0, 1.0, 2.0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditRottran() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(0, 45, 0);
	m.Translate(3, 0, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditTranrot() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Translate(3, 0, 0);
	m.Rotate(0, 45, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_AFTER);
}

void CMainFrame::OnEditReset() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_REPLACE);
}

void CMainFrame::OnEditRobjx() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(45, 0, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_BEFORE);
}

void CMainFrame::OnEditRobjy() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(0, 45, 0);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_BEFORE);
}

void CMainFrame::OnEditRobjz() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.Rotate(0, 0, 45);
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_BEFORE);
}

void CMainFrame::OnEditShear() 
{
	if (!m_pCurShape) return;
	C3dMatrix m;
	m.m_12 = 2;
	m_pCurShape->AddTransform(m, D3DRMCOMBINE_BEFORE);
}
